@@ -5,7 +5,6 @@ __all__ = ['request', 'HttpClient']
 import asyncio
 import collections
 import http.cookies
-import inspect
 import io
 import json
 import mimetypes
@@ -377,7 +376,7 @@ class ClientRequest:
         elif isinstance(data, (asyncio.StreamReader, streams.DataQueue)):
             self.body = data
 
-        elif inspect.isgenerator(data):
+        elif asyncio.iscoroutine(data):
             self.body = data
             if 'CONTENT-LENGTH' not in self.headers and self.chunked is None:
                 self.chunked = True
@@ -403,9 +402,9 @@ class ClientRequest:
             self.body = data(self.encoding)
 
             if 'CONTENT-TYPE' not in self.headers:
-                self.headers['CONTENT-TYPE'] = data.contenttype
+                self.headers['CONTENT-TYPE'] = data.content_type
 
-            if data.is_form_data():
+            if data.is_multipart:
                 self.chunked = self.chunked or 8196
             else:
                 if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
@@ -447,7 +446,7 @@ class ClientRequest:
             yield from self._continue
 
         try:
-            if inspect.isgenerator(self.body):
+            if asyncio.iscoroutine(self.body):
                 exc = None
                 value = None
                 stream = self.body
@@ -515,7 +514,12 @@ class ClientRequest:
                     'Can not write request body for %s' % self.url))
         else:
             try:
-                request.write_eof()
+                ret = request.write_eof()
+                # NB: in asyncio 3.4.1+ StreamWriter.drain() is coroutine
+                # see bug #170
+                if (asyncio.iscoroutine(ret) or
+                        isinstance(ret, asyncio.Future)):
+                    yield from ret
             except Exception as exc:
                 reader.set_exception(
                     aiohttp.ClientConnectionError(
@@ -712,8 +716,7 @@ class ClientResponse:
         if decode:
             warnings.warn(
                 '.read(True) is deprecated. use .json() instead',
-                DeprecationWarning
-            )
+                DeprecationWarning)
             return (yield from self.json())
 
         return data
@@ -723,8 +726,7 @@ class ClientResponse:
         """Read response payload and then close response."""
         warnings.warn(
             'read_and_close is deprecated, use .read() instead',
-            DeprecationWarning
-        )
+            DeprecationWarning)
         return (yield from self.read(decode))
 
     def _get_encoding(self, encoding):
