@@ -80,15 +80,10 @@ def run_server(loop, *, listen_addr=('127.0.0.1', 0),
                     break
 
             if router is not None:
-                body = bytearray()
-                try:
-                    while True:
-                        body.extend((yield from payload.read()))
-                except aiohttp.EofStream:
-                    pass
+                body = yield from payload.read()
 
                 rob = router(
-                    self, properties, self.transport, message, bytes(body))
+                    self, properties, self.transport, message, body)
                 rob.dispatch()
 
             else:
@@ -243,6 +238,8 @@ class Router:
         }
         if body:  # pragma: no cover
             resp['content'] = body
+        else:
+            resp['content'] = self._body.decode('utf-8', 'ignore')
 
         ct = self._headers.get('content-type', '').lower()
 
@@ -271,8 +268,10 @@ class Router:
                             msg.get('content-disposition', ''))
                         params['data'] = msg.get_payload()
                         params['content-type'] = msg.get_content_type()
+                        cte = msg.get('content-transfer-encoding')
+                        if cte is not None:
+                            resp['content-transfer-encoding'] = cte
                         resp['multipart-data'].append(params)
-
         body = json.dumps(resp, indent=4, sort_keys=True)
 
         # default headers
